from typing import List, Optional
from fastapi import Depends, FastAPI, Form, HTTPException
from sqlalchemy.orm import Session
import hashlib

from . import crud, models, schemas
from .database import SessionLocal, engine

models.Base.metadata.create_all(bind=engine)

app = FastAPI()

def get_db():
    db = SessionLocal()
    try:
        yield db
    finally:
        db.close()

@app.get("/")
async def root():
    return {"message": "Hello World"}

@app.post("/login/", response_model=schemas.User)
async def login(email: str = Form(...), password: str = Form(...), db:Session = Depends(get_db)):
    user = crud.get_user_by_email(db, email=email)
    if user is None:
        raise HTTPException(status_code=400, detail="Email not found")
    md5 = hashlib.md5()
    md5.update(password.encode('utf-8'))
    passwordMD5 = md5.hexdigest()
    if passwordMD5 != user.password:
        raise HTTPException(status_code=400, detail="Email or password is wrong")
    return user

@app.post("/users", response_model=schemas.User)
def create_user(user: schemas.UserCreate, db:Session = Depends(get_db)):
    db_user = crud.get_user_by_email(db, email=user.email)
    if db_user:
        raise HTTPException(status_code=400, detail="Email alread register")
    return crud.create_user(db=db, user=user)

@app.get("/users/", response_model=List[schemas.User])
def read_users(skip: int = 0, limit: int = 100, db: Session = Depends(get_db)):
    users = crud.get_users(db, skip=skip, limit=limit)
    return users

@app.get("/users/{user_id}", response_model=schemas.User)
def read_user(user_id: int, db: Session = Depends(get_db)):
    db_user = crud.get_user(db, user_id=user_id)
    if db_user is None:
        raise HTTPException(status_code=404, detail="User not found")
    return db_user

@app.post("/users/{user_id}/items", response_model=schemas.Item)
def read_items(skip: int = 0, limit: int = 100, db:Session = Depends(get_db)):
    items = crud.get_items(db, skip=skip, limit=limit)
    return items
